/*
    Drools4 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools4;

import org.drools.RuleBase;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.commons.AdviceAdapter;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.Type;

/**
 * @author mbo
 * @since 1.1.0
 */
class DroolsAddConstructorMethodVisitor extends AdviceAdapter {
	
	private String parent;
			
	private String droolsName;
	
	private String droolsRule;

	public DroolsAddConstructorMethodVisitor(
			MethodVisitor mv, 
			int access,
			String name, 
			String desc,
			String parent,
			String droolsName,
			String droolsRule) {
		super(mv, access, name, desc);
		this.parent=parent;
		this.droolsName=droolsName;
		this.droolsRule=droolsRule;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.commons.AdviceAdapter#onMethodEnter()
	 */
	@Override
	protected void onMethodEnter() {
		this.visitVarInsn(Opcodes.ALOAD,0);
		this.visitInsn(Opcodes.DUP);
		this.visitInsn(Opcodes.DUP);
		this.visitInsn(Opcodes.ICONST_0);
		this.visitFieldInsn(Opcodes.PUTFIELD,Type.getObjectType(parent).getInternalName(), droolsName, Type.BOOLEAN_TYPE.getDescriptor());
		this.visitTypeInsn(Opcodes.NEW,Type.getType(DroolsProvider.class).getInternalName());
		this.visitInsn(Opcodes.DUP);		
		this.visitMethodInsn(Opcodes.INVOKESPECIAL, Type.getType(DroolsProvider.class).getInternalName(), "<init>","()V");
		this.visitInsn(Opcodes.SWAP);
		this.visitMethodInsn(Opcodes.INVOKEVIRTUAL, Type.getObjectType(parent).getInternalName(), "getClass", "()Ljava/lang/Class;");
		this.visitMethodInsn(Opcodes.INVOKEVIRTUAL, Type.getType(DroolsProvider.class).getInternalName(), "getRuleBase","(Ljava/lang/Class;)Lorg/drools/RuleBase;");
		this.visitFieldInsn(Opcodes.PUTFIELD,Type.getObjectType(parent).getInternalName(), droolsRule, Type.getType(RuleBase.class).getDescriptor());
	}


	@Override
	public void visitMaxs(int maxStack, int maxLocals) {
		super.visitMaxs(maxStack+4, maxLocals);
	}

}
