/*
    Drools4 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools4;

import java.io.File;

import org.apache.maven.execution.MavenSession;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.shared.filtering.MavenResourcesFiltering;
import org.codehaus.plexus.util.DirectoryScanner;


/**
 * This goal compile drools
 * @author mbo
 * @since 1.1.0
 * @goal drools-postprocessor
 * @phase process-classes
 * @requiresDependencyResolution runtime
 */
public class DroolsClassPostProcessor extends AbstractMojo {

	@Override
	public void execute() throws MojoExecutionException, MojoFailureException {
		if (getLog().isDebugEnabled()) getLog().debug("starting drools post-compilation");
		DirectoryScanner scanner = new DirectoryScanner();
		if (!getInputDirectory().exists()) {
			getLog().warn("Skipping not existing directory "+getInputDirectory());
			return;
		}
		scanner.setBasedir(getInputDirectory());
		scanner.setIncludes(new String[] { "**/*"+getExtension(),"*"+getExtension() });
		scanner.scan();
		boolean ok=true;
		for (String file : scanner.getIncludedFiles()) {
			File sfile = new File(getInputDirectory(), file);
			DroolsClassProcessor.ClassProcessor(getLog(), sfile);
		}
		if (!ok) {
			throw new MojoExecutionException("Error of the drools");
		}
	}

	/**
	 * @parameter expression="${session}"
	 * 
	 * @readonly
	 * @required
	 */
	protected MavenSession session;
	
	/**
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    protected MavenProject project;
    
    /**
     * The input directory from where to copy the rules.
     * 
     * @parameter expression="${project.basedir}/target/classes"
     * @required
     */
    private File inputDirectory;
    
    /**
     * The default extension for drools file
     * 
     * @parameter expression=".class"
     * @required
     */
    private String extension;
    
    /**
     * 
     * @component role="org.apache.maven.shared.filtering.MavenResourcesFiltering" role-hint="default"
     * @required
     */    
    protected MavenResourcesFiltering mavenResourcesFiltering;

	public File getInputDirectory() {
		return inputDirectory;
	}

	public void setInputDirectory(File inputDirectory) {
		this.inputDirectory = inputDirectory;
	}

	public String getExtension() {
		return extension;
	}

	public void setExtension(String extension) {
		this.extension = extension;
	}

}
